#!/usr/bin/python
# -*- coding: utf-8 -*-
import time
from .cal_crawler import CalCrawler
from .healthy_checker import HealthyChecker
from .common_utils import CommonUtils


class AIMonitor(object):
    @staticmethod
    def __monitor_live_box(host_name, start_time, end_time, pool_name="risktxncomputeserv"):
        cal_log_data = CalCrawler.crawl_data(host_name, start_time, end_time, pool_name=pool_name)
        check_result = HealthyChecker.abnormal_check(cal_log_data)
        print("Check box %s from %s to %s %s\n\n" % (host_name, CommonUtils.timestamp_to_data_str(start_time), CommonUtils.timestamp_to_data_str(end_time), HealthyChecker.result_analysis(check_result)))

    @staticmethod
    def monitor_last_x_hour(host_name, hours=24, pool_name="risktxncomputeserv"):
        current_timestamp = int(time.time())
        last_x_hours_timestamp = current_timestamp - hours * 60 * 60
        AIMonitor.__monitor_live_box(host_name, last_x_hours_timestamp, current_timestamp, pool_name)

    @staticmethod
    def monitor_last_x_day(host_name, days=7, pool_name="risktxncomputeserv"):
        current_timestamp = int(time.time())
        last_x_days_timestamp = current_timestamp - days * 24 * 60 * 60
        AIMonitor.__monitor_live_box(host_name, last_x_days_timestamp, current_timestamp, pool_name)

    @staticmethod
    def monitor_one_box(one_box_list, one_box_start_time, time_format="%Y-%m-%d %H:%M:%S", pool_name="risktxncomputeserv"):
        current_timestamp = int(time.time())
        start_timestamp = CommonUtils.date_str_to_timestamp(one_box_start_time, time_format)
        for host_name in one_box_list:
            AIMonitor.__monitor_live_box(host_name, start_timestamp, current_timestamp, pool_name)


if __name__ == "__main__":
    AIMonitor.monitor_one_box(["dcg12risktxncomputeserv9421"], "2018-07-11 01:22:33", pool_name="risktxncomputeserv")





#!/usr/bin/python
# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import datetime
import time
from .cal_crawler import CalCrawler
from .common_utils import CommonUtils


class CalPlot(object):
    @staticmethod
    def __plot(history_data):
        fig = plt.figure(figsize=(15, 8))
        sub_fig_num = len(history_data)
        base_value = sub_fig_num * 100 + 10 + sub_fig_num
        for data_type, data_list in history_data.items():
            ax = fig.add_subplot(base_value)
            base_value -= 1
            x, y = zip(*data_list)
            x = [datetime.datetime.fromtimestamp(ts) for ts in x]
            ax.plot(x, y)
            ax.set_title(data_type.name)
        plt.gca().xaxis.set_major_formatter(mdates.DateFormatter('%m/%d/%Y %H:%M'))
        plt.gca().xaxis.set_major_locator(mdates.HourLocator())
        plt.gcf().autofmt_xdate()
        plt.show()

    @staticmethod
    def show_instance_history(host_name, start_time_str, end_time_str=None, time_str_format="%Y-%m-%d %H:%M:%S"):
        if end_time_str is None:
            end_time = int(time.time())
        else:
            end_time = CommonUtils.date_str_to_timestamp(end_time_str, time_str_format)
        start_time = CommonUtils.date_str_to_timestamp(start_time_str, time_str_format)

        result = CalCrawler.crawl_data(host_name, start_time, end_time)
        history_data = {}
        for data_type, data_list in result.items():
            if len(data_list) > 0 and 'DataPoints' in data_list[0] and len(data_list[0]['DataPoints']) > 0:
                history_data[data_type] = data_list[0]['DataPoints']
        CalPlot.__plot(history_data)


if __name__ == "__main__":
    CalPlot.show_instance_history("dcg12risktxncomputeserv6950", "2018-07-09 10:12:12", "2018-07-10 10:12:12")






import setuptools


# Change this line to the module name you want to create
__title__ = "cal_ai"
__summary__ = "An end-to-end machine learning and data mining framework on Hadoop CAL."
__uri__ = "https://github.paypal.com/haifwu/LiveBoxMonitor"

__author__ = "Wu Haifeng"
__email__ = "wuhaifengdhu@163.com"

# Change the version when you want to release new version.
__version__ = "0.2.0.4"


with open("README.rst", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name=__title__,
    version=__version__,
    author=__author__,
    author_email=__email__,
    description=__summary__,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=__uri__,
    packages=setuptools.find_packages(),
    package_data={'cal_ai': ['README.rst', 'LICENCE']},
    classifiers=(
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ),
)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import time
import logging
import urllib, json
from threading import Thread
from .data_template import UrlTemplate, DataType
from .common_utils import CommonUtils


class CalCrawler(object):
    @staticmethod
    def __crawl(data_url, data_type, current_part_num, history_data, retry=5):
        start_time = time.time()
        logging.debug("%s thread start at time: %s" % (data_type, CommonUtils.timestamp_to_data_str(start_time)))
        logging.debug(data_url)

        for i in range(retry):
            try:
                response = urllib.urlopen(data_url)
                data = json.loads(response.read())
                history_data[data_type][current_part_num] = data
                if i > 0:
                    logging.warn("Get good result for %s through %d retry after IOException." % (str(data_type), i))
                break
            except IOError:
                # Retry when IO Error Happen
                if i == retry - 1:
                    logging.error("Curl data type %s failed after %d time retry, url %s" % (data_type, i, data_url))
            time.sleep(3)

        finish_time = time.time()
        logging.debug("%s thread finished at time: %s, total take %s second" % (data_type, CommonUtils.timestamp_to_data_str(finish_time), str(finish_time - start_time)))

    @staticmethod
    def __merge_history_data(history_data):
        merged_history_data = {}
        for data_type in DataType:
            data_type_dict = history_data[data_type]
            if not data_type_dict:
                logging.warn("Data crawl for type %s is None" % data_type.name)
            elif len(data_type_dict) != max(data_type_dict.keys()):
                logging.error("Crawl data part missing for type %s, collected parts: %s" % (data_type.name, str(data_type.keys())))
            else:
                merged_history_data[data_type] = []
                for i in range(1, len(data_type_dict) + 1):
                    logging.info("start merge %d for type %s" % (i, data_type.name))
                    if data_type_dict[i]:
                        if not merged_history_data[data_type]:
                            merged_history_data[data_type] = data_type_dict[i]
                        else:
                            merged_history_data[data_type][0]['DataPoints'].extend(data_type_dict[i][0]['DataPoints'])
        return merged_history_data

    @staticmethod
    def crawl_data(host_name, start_time, end_time, column_name=None, pool_name="risktxncomputeserv", output_type="json", use_cache=True, max_wait_second=300):
        if column_name is None:
            column_name = CommonUtils.get_column_name_from_host_name(host_name)
            logging.debug("Get column name %s from host %s" % (column_name, host_name))

        thread_list = []
        history_data = {}
        for data_type in DataType:
            history_data[data_type] = {}
            current_part_num = 1
            while start_time + 24 * 60 * 60 < end_time:
                CalCrawler.__new_crawl_thread(data_type, start_time, start_time + 24 * 60 * 60, pool_name, column_name,
                                         host_name, output_type, use_cache, current_part_num, thread_list, history_data)
                start_time += 24 * 60 * 60
                current_part_num += 1
            CalCrawler.__new_crawl_thread(data_type, start_time, end_time, pool_name, column_name, host_name, output_type,
                                          use_cache, current_part_num, thread_list, history_data)
        for thread in thread_list:
            thread.join(max_wait_second)

        return CalCrawler.__merge_history_data(history_data)

    @staticmethod
    def __new_crawl_thread(data_type, start_time, end_time, pool_name, column_name, host_name, output_type, use_cache,
                           current_part_num, thread_list, history_data):
        data_url = UrlTemplate.render(data_type, start_time, end_time, pool_name, column_name, host_name, output_type, use_cache)
        thread = Thread(target=CalCrawler.__crawl, args=[data_url, data_type, current_part_num, history_data])
        thread.daemon = True
        thread.start()
        thread_list.append(thread)


if __name__ == "__main__":
    host_name = "dcg13risktxncomputeserv6056"
    result = CalCrawler.crawl_data(host_name, 1530979200, 1531324800)
    for key, value in result.items():
        print(key, value)


